/*****************************************************************************
*
* File Name  : wm_wdrt_demo.c
*
* Description: wifi direct rx or tx demo
*
* Copyright (c) 2014 Winner Micro Electronic Design Co., Ltd.
* All rights reserved.
*
* Author     : 
*
* Date : 2019-5-6
*****************************************************************************/
#include <string.h>
#include "wm_include.h"
#include "wm_ieee80211.h"
#include "tls_common.h"
#include "tls_wireless.h"
#include "wm_wdrt_demo.h"

extern u8 *wpa_supplicant_get_mac(void);
extern struct tls_wif * tls_get_wif_data(void);

const static u8 wm_wdrt_eth_llc[8] = {0xAA, 0xAA, 0x03, 0x28, 0x6D, 0xCD, 0xEE, 0xEF};

static int wm_wdrt_last_seq = -1;
static u8 wm_wdrt_last_sa_mac[ETH_ALEN];
static wm_wdrt_rx_callback wm_wdrt_demo_rx_callback = NULL;

static void wm_wdrt_demo_recv_callback(u8* data, u32 data_len)
{
    struct ieee80211_hdr_3addr *hdr;
    u8 *mac = wpa_supplicant_get_mac();

    if (data_len < (sizeof(struct ieee80211_hdr_3addr) + sizeof(wm_wdrt_eth_llc)))
        return;

    hdr = (struct ieee80211_hdr_3addr *)data;

    if (ieee80211_has_fromds(hdr->frame_control) ||
        ieee80211_has_tods(hdr->frame_control))
        return;

    if (!ieee80211_is_data(hdr->frame_control))
        return;

    if (ieee80211_is_data_qos(hdr->frame_control))
        return;

    if (compare_ether_addr(hdr->addr1, mac) &&
        compare_ether_addr(hdr->addr2, hdr->addr3))
        return;

    if (ieee80211_has_retry(hdr->frame_control))
    {
        if (-1 != wm_wdrt_last_seq)
        {
            if ((hdr->seq_ctrl == wm_wdrt_last_seq) &&
                (0 == compare_ether_addr(hdr->addr2, wm_wdrt_last_sa_mac)))
                return;
        }
    }

    wm_wdrt_last_seq = hdr->seq_ctrl;
    memcpy(wm_wdrt_last_sa_mac, hdr->addr2, ETH_ALEN);

    data += sizeof(struct ieee80211_hdr_3addr);
    data_len -= sizeof(struct ieee80211_hdr_3addr);

    if (memcmp(data, wm_wdrt_eth_llc, sizeof(wm_wdrt_eth_llc)))
        return;

    data += sizeof(wm_wdrt_eth_llc);
    data_len -= sizeof(wm_wdrt_eth_llc);

    wm_wdrt_demo_rx_callback(data, data_len);

    return;
}

static void wm_wdrt_demo_entry_tx_loop(u8 *da,
                                       enum tls_wifi_tx_rate tx_rate,
                                       u8 *data, u16 data_len)
{
    int ret;
    u8 *mac = wpa_supplicant_get_mac();
    struct tls_wifi_tx_rate_t tx;
    struct ieee80211_hdr_3addr *hdr;
    u8 *wpkt;
    u16 wlen;

    wlen = sizeof(struct ieee80211_hdr_3addr) + sizeof(wm_wdrt_eth_llc) + data_len;
    wpkt = tls_mem_alloc(wlen);
    if (NULL == wpkt)
    {
        printf("failed to malloc pkt buf\r\n");
        return;
    }

    memset(wpkt, 0, wlen);
    hdr = (struct ieee80211_hdr_3addr *)wpkt;

    hdr->frame_control = host_to_le16(IEEE80211_FTYPE_DATA | IEEE80211_STYPE_DATA);

    memcpy(hdr->addr1, da, ETH_ALEN);
    memcpy(hdr->addr2, mac, ETH_ALEN);
    memcpy(hdr->addr3, mac, ETH_ALEN);

    memcpy(wpkt + sizeof(struct ieee80211_hdr_3addr), wm_wdrt_eth_llc, sizeof(wm_wdrt_eth_llc));
    memcpy(wpkt + sizeof(struct ieee80211_hdr_3addr) + sizeof(wm_wdrt_eth_llc), data, data_len);

    tx.tx_rate = tx_rate;
    tx.tx_gain = tls_wifi_get_tx_gain_max(tx_rate);

    while (1)
    {
        ret = tls_wifi_send_data(NULL, wpkt, wlen, &tx);
        if (ret)
            printf("failed to send data\r\n");
    }

    //tls_mem_free(wpkt);/* unreachable */
}

void wm_wdrt_demo(wm_wdrt_cfg *cfg)
{
	printf("\n wifi direct %s demo\n", (cfg->mode == WM_WDRT_MODE_RX) ? "rx" : "tx");

    if (0  != cfg->work_channel)
        tls_wifi_change_chanel(cfg->work_channel - 1);

    if (WM_WDRT_MODE_TX == cfg->mode)
    {
        wm_wdrt_demo_entry_tx_loop(cfg->da, cfg->tx_rate, cfg->tx_data, cfg->tx_data_len);
    }
    else if (WM_WDRT_MODE_RX == cfg->mode)
    {
        wm_wdrt_demo_rx_callback = cfg->rx_callback;
        tls_wl_if_set_mode(tls_get_wif_data(), IEEE80211_MODE_IBSS);
        tls_wifi_data_recv_cb_register((tls_wifi_data_recv_callback)wm_wdrt_demo_recv_callback);
    }

    return;
}

